@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-gst.c,v 1.96 2004/10/11 15:53:13 rbultje Exp $
+ * $Id: bacon-video-widget-gst.c,v 1.99 2004/10/25 16:12:05 rbultje Exp $
  *
  */
 
@@ -871,9 +871,14 @@ got_error (GstElement *play, GstElement *orig, GError *error,
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 
-  /* the handler will catch it */
-  if (GST_STATE (play) != GST_STATE_PLAYING)
+  /* since we're opening, we will never enter the mainloop
+   * until we return, so setting an idle handler doesn't
+   * help... Anyway, let's prepare a message. */
+  if (GST_STATE (play) != GST_STATE_PLAYING) {
+    g_free (bvw->priv->last_error_message);
+    bvw->priv->last_error_message = g_strdup (error->message);
     return;
+  }
   
   signal = g_new0 (BVWSignal, 1);
   signal->signal_id = ASYNC_ERROR;
@@ -1910,9 +1915,26 @@ bacon_video_widget_get_metadata_int (BaconVideoWidget * bvw,
       else
         integer = bvw->priv->video_fps;
       break;
-    case BVW_INFO_BITRATE:
+    case BVW_INFO_BITRATE: {
+      gint num, t;
+
       integer = 0;
+      for (num = 0; ; num++) {
+        if (!gst_tag_list_get_uint_index (bvw->priv->tagcache,
+					  GST_TAG_BITRATE, num, &t))
+          break;
+        integer += t;
+      }
+      for (num = 0; ; num++) {
+        if (!gst_tag_list_get_uint_index (bvw->priv->tagcache,
+					  GST_TAG_NOMINAL_BITRATE, num, &t))
+          break;
+        integer += t;
+      }
+      /* kbps */
+      integer /= 1000;
       break;
+    }
     default:
       g_assert_not_reached ();
     }
