@@ -24,7 +24,6 @@
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <errno.h>
 #include <mntent.h>
 #include <string.h>
@@ -41,14 +40,17 @@
 
 typedef struct _CdCache {
   /* device node and mountpoint */
-  gchar *device, *mountpoint;
+  char *device, *mountpoint;
   GnomeVFSDrive *drive;
 
   /* file descriptor to the device */
-  gint fd;
+  int fd;
 
   /* capabilities of the device */
-  gint cap;
+  int cap;
+
+  /* if we're checking a media, or a dir */
+  gboolean is_media;
 
   /* indicates if we mounted this mountpoint ourselves or if it
    * was already mounted. */
@@ -56,20 +58,41 @@ typedef struct _CdCache {
   gboolean mounted;
 } CdCache;
 
+/* 
+ * Resolve relative paths
+ */
+
+static char *
+totem_disc_resolve_link (const char *dev, const char *buf)
+{
+  char *parent, *new;
+
+  /* is it an absolute path? */
+  if (g_path_is_absolute (buf) != FALSE)
+    return g_strdup (buf);
+
+  parent = g_path_get_dirname (dev);
+  new = g_build_filename (parent, buf, NULL);
+  g_free (parent);
+
+  return new;
+}
+
 /*
  * So, devices can be symlinks and that screws up.
  */
 
-static gchar *
-get_device (const gchar *device,
+static char *
+get_device (const char *device,
 	    GError     **error)
 {
-  gchar buf[256];
-  gchar *dev = g_strdup (device);
+  char *buf;
+  char *dev = g_strdup (device);
   struct stat st;
-  gint read;
 
   while (1) {
+    char *new;
+
     if (lstat (dev, &st) != 0) {
       g_set_error (error, 0, 0,
           _("Failed to find real device node for %s: %s"),
@@ -81,29 +104,42 @@ get_device (const gchar *device,
     if (!S_ISLNK (st.st_mode))
       break;
 
-    if ((read = readlink (dev, buf, 255)) < 0) {
+    if (!(buf = g_file_read_link (dev, NULL))) {
       g_set_error (error, 0, 0,
           _("Failed to read symbolic link %s: %s"),
           dev, g_strerror (errno));
       g_free (dev);
       return NULL;
     }
+    new = totem_disc_resolve_link (dev, buf);
     g_free (dev);
-    dev = g_strndup (buf, read);
+    g_free (buf);
+    dev = new;
   }
 
   return dev;
 }
 
 static CdCache *
-cd_cache_new (const gchar *dev,
+cd_cache_new (const char *dev,
 	      GError     **error)
 {
   CdCache *cache;
-  gchar *mountpoint = NULL, *device;
+  char *mountpoint = NULL, *device;
   GnomeVFSVolumeMonitor *mon;
   GnomeVFSDrive *drive = NULL;
   GList *list, *or;
+  gboolean is_dir;
+
+  is_dir = g_file_test (dev, G_FILE_TEST_IS_DIR);
+  if (is_dir) {
+    cache = g_new0 (CdCache, 1);
+    cache->mountpoint = g_strdup (dev);
+    cache->fd = -1;
+    cache->is_media = FALSE;
+
+    return cache;
+  }
 
   /* retrieve mountpoint (/etc/fstab). We could also use HAL for this,
    * I think (gnome-volume-manager does that). */
@@ -112,7 +148,7 @@ cd_cache_new (const gchar *dev,
   mon = gnome_vfs_get_volume_monitor ();
   for (or = list = gnome_vfs_volume_monitor_get_connected_drives (mon);
        list != NULL; list = list->next) {
-    gchar *pdev, *pdev2;
+    char *pdev, *pdev2;
 
     drive = list->data;
     if (!(pdev = gnome_vfs_drive_get_device_path (drive)))
@@ -124,7 +160,7 @@ cd_cache_new (const gchar *dev,
     g_free (pdev);
 
     if (strcmp (pdev2, device) == 0) {
-      gchar *mnt;
+      char *mnt;
 
       mnt = gnome_vfs_drive_get_activation_uri (drive);
       g_free (pdev2);
@@ -155,6 +191,7 @@ cd_cache_new (const gchar *dev,
   cache->fd = -1;
   cache->self_mounted = FALSE;
   cache->drive = drive;
+  cache->is_media = TRUE;
 
   return cache;
 }
@@ -165,6 +202,12 @@ cd_cache_open_device (CdCache *cache,
 {
   int drive, err;
 
+  /* not a medium? */
+  if (cache->is_media == FALSE) {
+    cache->cap = CDC_DVD;
+    return TRUE;
+  }
+
   /* already open? */
   if (cache->fd > 0)
     return TRUE;
@@ -195,7 +238,7 @@ cd_cache_open_device (CdCache *cache,
 
   /* is there a disc in the tray? */
   if ((drive = ioctl (cache->fd, CDROM_DRIVE_STATUS, NULL)) != CDS_DISC_OK) {
-    const gchar *drive_s;
+    const char *drive_s;
 
     close (cache->fd);
     cache->fd = -1;
@@ -234,7 +277,7 @@ cd_cache_open_mountpoint (CdCache *cache,
 			  GError **error)
 {
   /* already opened? */
-  if (cache->mounted)
+  if (cache->mounted || cache->is_media == FALSE)
     goto opendir;
 
   /* check for mounting - assume we'll mount ourselves */
@@ -242,8 +285,8 @@ cd_cache_open_mountpoint (CdCache *cache,
 
   /* mount if we have to */
   if (cache->self_mounted) {
-    gchar *command;
-    gint status;
+    char *command;
+    int status;
 
     command = g_strdup_printf ("mount %s", cache->mountpoint);
     if (!g_spawn_command_line_sync (command,
@@ -271,7 +314,7 @@ cd_cache_free (CdCache *cache)
 {
   /* umount if we mounted */
   if (cache->self_mounted && cache->mounted) {
-    gchar *command;
+    char *command;
 
     command = g_strdup_printf ("umount %s", cache->mountpoint);
     g_spawn_command_line_sync (command, NULL, NULL, NULL, NULL);
@@ -295,8 +338,12 @@ cd_cache_disc_is_cdda (CdCache *cache,
 		       GError **error)
 {
   MediaType type = MEDIA_TYPE_DATA;
-  gint disc;
-  const gchar *disc_s;
+  int disc;
+  const char *disc_s;
+
+  /* We can't have audio CDs on disc, yet */
+  if (cache->is_media == FALSE)
+    return type;
 
   /* open disc and open mount */
   if (!cd_cache_open_device (cache, error))
@@ -361,7 +408,7 @@ cd_cache_disc_is_vcd (CdCache *cache,
   while ((name = g_dir_read_name (dir)) != NULL) {
     if (g_ascii_strcasecmp (name, "mpegav") == 0) {
       GDir *subdir;
-      gchar *subdirname = g_build_path (G_DIR_SEPARATOR_S,
+      char *subdirname = g_build_path (G_DIR_SEPARATOR_S,
 		      	       cache->mountpoint, name, NULL);
 
       have_mpegav = TRUE;
@@ -370,8 +417,10 @@ cd_cache_disc_is_vcd (CdCache *cache,
         return MEDIA_TYPE_ERROR;
       }
       while ((name = g_dir_read_name (subdir)) != NULL) {
-        if (g_ascii_strcasecmp (name, "avseq01.dat") == 0)
+        if (g_ascii_strcasecmp (name, "avseq01.dat") == 0) {
 	  have_avseq = TRUE;
+	  break;
+	}
       }
       g_dir_close (subdir);
       g_free (subdirname);
@@ -389,8 +438,8 @@ cd_cache_disc_is_dvd (CdCache *cache,
 		      GError **error)
 {
   GDir *dir;
-  const gchar *name;
-  gboolean have_vts = FALSE, have_vtsifo = FALSE, have_ats = FALSE;
+  const char *name;
+  gboolean have_vts = FALSE, have_vtsifo = FALSE;
 
   /* open disc, check capabilities and open mount */
   if (!cd_cache_open_device (cache, error))
@@ -404,7 +453,7 @@ cd_cache_disc_is_dvd (CdCache *cache,
   while ((name = g_dir_read_name (dir)) != NULL) {
     if (g_ascii_strcasecmp (name, "VIDEO_TS") == 0) {
       GDir *subdir;
-      gchar *subdirname = g_build_path (G_DIR_SEPARATOR_S,
+      char *subdirname = g_build_path (G_DIR_SEPARATOR_S,
                                cache->mountpoint, name, NULL);
 
       have_vts = TRUE;
@@ -413,23 +462,72 @@ cd_cache_disc_is_dvd (CdCache *cache,
         return MEDIA_TYPE_ERROR;
       }
       while ((name = g_dir_read_name (subdir)) != NULL) {
-        if (g_ascii_strcasecmp (name, "VIDEO_TS.IFO") == 0)
+        if (g_ascii_strcasecmp (name, "VIDEO_TS.IFO") == 0) {
           have_vtsifo = TRUE;
+	  break;
+	}
       }
       g_dir_close (subdir);
       g_free (subdirname);
-    } else if (g_ascii_strcasecmp (name, "AUDIO_TS") == 0) {
-      have_ats = TRUE;
+      break;
     }
   }
   g_dir_close (dir);
 
-  return (have_vts && have_ats && have_vtsifo) ?
+  return (have_vts && have_vtsifo) ?
       MEDIA_TYPE_DVD : MEDIA_TYPE_DATA;
 }
 
 MediaType
-cd_detect_type (const gchar *device,
+cd_detect_type_from_dir (const char *dir, char **url, GError **error)
+{
+  CdCache *cache;
+  MediaType type;
+
+  g_return_val_if_fail (dir != NULL, MEDIA_TYPE_ERROR);
+
+  if (dir[0] != '/' && g_str_has_prefix (dir, "file://") == FALSE)
+    return MEDIA_TYPE_ERROR;
+
+  if (!(cache = cd_cache_new (dir, error)))
+    return MEDIA_TYPE_ERROR;
+  if ((type = cd_cache_disc_is_vcd (cache, error)) == MEDIA_TYPE_DATA &&
+      (type = cd_cache_disc_is_dvd (cache, error)) == MEDIA_TYPE_DATA) {
+    /* crap, nothing found */
+    cd_cache_free (cache);
+    return type;
+  }
+  cd_cache_free (cache);
+
+  if (url == NULL) {
+    return type;
+  }
+
+  if (type == MEDIA_TYPE_DVD) {
+    if (g_str_has_prefix (dir, "file://") != FALSE) {
+      char *local;
+      local = g_filename_from_uri (dir, NULL, NULL);
+      *url = g_strdup_printf ("dvd://%s", local);
+      g_free (local);
+    } else {
+      *url = g_strdup_printf ("dvd://%s", dir);
+    }
+  } else if (type == MEDIA_TYPE_VCD) {
+    if (g_str_has_prefix (dir, "file://") != FALSE) {
+      char *local;
+      local = g_filename_from_uri (dir, NULL, NULL);
+      *url = g_strdup_printf ("vcd://%s", local);
+      g_free (local);
+    } else {
+      *url = g_strdup_printf ("vcd://%s", dir);
+    }
+  }
+
+  return type;
+}
+
+MediaType
+cd_detect_type (const char *device,
 		GError     **error)
 {
   CdCache *cache;
@@ -446,3 +544,7 @@ cd_detect_type (const gchar *device,
 
   return type;
 }
+
+/*
+ * vim: sw=2 ts=8 cindent noai bs=2
+ */
